/* $Id: 1$ */
/** @acpi.c
  *
  *  External function structure for ACPI
  *
  * netlabs.org confidential
  *
  * Copyright (c) 2006 netlabs.org
  *
  * Author: Pavel Shtemenko <pasha@paco.odessa.ua>
  *
  * All Rights Reserved
  *
  */
#ifndef __ACPIFUN_H__
#define __ACPIFUN_H__

#pragma pack(1)

#ifdef __cplusplus
extern "C" {
#endif


typedef ACPI_STATUS                  (*ACPIEVALUATEOBJECT         )(ACPI_HANDLE Object,ACPI_STRING  Pathname, ACPI_OBJECT_LIST *ParameterObjects, ACPI_BUFFER *ReturnObjectBuffer);
typedef ACPI_STATUS                  (*ACPIWALKNAMESPACE          )(ACPI_OBJECT_TYPE Type,ACPI_HANDLE StartObject,UINT32 MaxDepth,ACPI_WALK_CALLBACK UserFunction,void *Context,void **ReturnValue);
typedef ACPI_STATUS                  (*ACPIGETDEVICES             )(char *HID,ACPI_WALK_CALLBACK UserFunction,void *Context,void **ReturnValue);
typedef ACPI_STATUS                  (*ACPIGETNAME                )(ACPI_HANDLE Handle,UINT32 NameType,ACPI_BUFFER *RetPathPtr);
typedef ACPI_STATUS                  (*ACPIGETHANDLE              )(ACPI_HANDLE Parent,ACPI_STRING Pathname,ACPI_HANDLE *RetHandle);
typedef ACPI_STATUS                  (*ACPIGETOBJECTINFO          )(ACPI_HANDLE Handle,ACPI_BUFFER *ReturnBuffer);
typedef ACPI_STATUS                  (*ACPIGETTYPE                )(ACPI_HANDLE Object,ACPI_OBJECT_TYPE *OutType);
typedef ACPI_STATUS                  (*ACPIGETNEXTOBJECT          )(ACPI_OBJECT_TYPE Type, ACPI_HANDLE Parent, ACPI_HANDLE Child, ACPI_HANDLE *OutHandle);
typedef ACPI_STATUS                  (*ACPIGETPARENT              )(ACPI_HANDLE Object, ACPI_HANDLE *OutHandle);
typedef const char                  *(*ACPIFORMATEXEPTION         )(ACPI_STATUS Exception);
typedef int                          (*ACPIUTMEMCMP               )(const char  *Buffer1,const char *Buffer2,ACPI_SIZE Count);
typedef void                        *(*ACPIUTMEMSET               )(void *Dest, ACPI_NATIVE_UINT Value, ACPI_SIZE Count);
typedef void (ACPI_INTERNAL_VAR_XFACE *ACPIOSPRINTF               )(const char *Format, ...);
typedef ACPI_STATUS                  (*ACPIOSREADPCICONFIGURATION )(ACPI_PCI_ID *PciId,UINT32 Register, void         *Value, UINT32 Width);
typedef ACPI_STATUS                  (*ACPIOSWRITEPCICONFIGURATION)(ACPI_PCI_ID *PciId,UINT32 Register, ACPI_INTEGER  Value, UINT32 Width);
typedef void                        *(*ACPIOSALLOCATE             )(ACPI_SIZE Size);
typedef void                         (*ACPIOSFREE                 )(void *Memory);
typedef ACPI_STATUS                  (*ACPIGETCURRENTRESOURCES    )(ACPI_HANDLE DeviceHandle, ACPI_BUFFER *RetBuffer);
typedef ACPI_STATUS                  (*ACPIGETPOSSIBLERESOURCES   )(ACPI_HANDLE DeviceHandle, ACPI_BUFFER *RetBuffer);
typedef ACPI_STATUS                  (*ACPISETCURRENTRESOURCES    )(ACPI_HANDLE DeviceHandle, ACPI_BUFFER  *InBuffer);
typedef ACPI_STATUS                  (*ACPIENTERSLEEPSTATEPREP    )(UINT8 SleepState);
typedef ACPI_STATUS                  (*ACPIENTERSLEEPSTATE        )(UINT8 SleepState);
typedef ACPI_STATUS                  (*ACPIENTERSLEEPSTATES4BIOS  )(void);
typedef ACPI_STATUS                  (*ACPILEAVESLEEPSTATE        )(UINT8 SleepState);
typedef UINT32                       (*ACPIUTSTRTOUL              )(const char *String, char **Terminator, UINT32  Base);
typedef ACPI_STATUS                  (*SETTHROTTLINGORCSTATE      )(PTHROTTLING_C_STATE State);
typedef ACPI_STATUS                  (*ACPIGETTIMER               )(UINT32  *Ticks);
typedef ACPI_STATUS                  (*ACPIGETTIMERDURATION       )(UINT32  StartTicks, UINT32  EndTicks, UINT32 *TimeElapsed);
typedef ACPI_STATUS                  (*ACPIWALKRESOURCES          )(ACPI_HANDLE DeviceHandle, char *Name, ACPI_WALK_RESOURCE_CALLBACK UserFunction, void *Context);
// For run function/subprogramm at all CPU
typedef ACPI_STATUS ( APIENTRY        *ACPIRUNATALLCPU            )(void *Contex);
typedef ACPI_STATUS ( APIENTRY        *ACPIEXECSMPFUNCTION        )(ACPIRUNATALLCPU Subprogram,UINT32 CPUNMask,UINT32 Flags,void *Context );
//
typedef int                          (*ACPIUTSTRNCMP              )(const char *String1, const char *String2, ACPI_SIZE Count);
typedef void *                       (*ACPIUTMEMCPY               )(void *Dest, const void *Src, ACPI_SIZE  Count);
typedef ULONG                        (PSDCALL *PSDAPPCOM          )(APPCOMMPAR *par);
typedef ACPI_STATUS ( APIENTRY        *FINDPCIDEVICE              )(PKNOWNDEVICE f);

#ifdef __cplusplus
        }
#endif

ACPI_STATUS APIENTRY AcpiExecSMPFunction(ACPIRUNATALLCPU Subprogram,UINT32 CPUNMask,UINT32 Flags,void *Context );

typedef struct _ACPI_FUNCTION_
{
    ACPIEXECSMPFUNCTION         AcpiExecSMPFunction;                    //
    ACPIUTSTRNCMP               AcpiUtStrncmp;
    ACPIEVALUATEOBJECT          AcpiEvaluateObject;
    ACPIWALKNAMESPACE           AcpiWalkNamespace;
    ACPIGETDEVICES              AcpiGetDevices;
    ACPIGETNAME                 AcpiGetName;
    ACPIGETHANDLE               AcpiGetHandle;
    ACPIGETOBJECTINFO           AcpiGetObjectInfo;
    ACPIGETTYPE                 AcpiGetType;
    ACPIGETNEXTOBJECT           AcpiGetNextObject;
    ACPIGETPARENT               AcpiGetParent;
    ACPIFORMATEXEPTION          AcpiFormatException;
    ACPIUTMEMCMP                AcpiUtMemcmp;
    ACPIUTMEMSET                AcpiUtMemset;
    ACPIOSPRINTF                AcpiOsPrintf;
    ACPIOSREADPCICONFIGURATION  AcpiOsReadPciConfiguration;
    ACPIOSWRITEPCICONFIGURATION AcpiOsWritePciConfiguration;
    ACPIOSALLOCATE              AcpiOsAllocate;
    ACPIOSFREE                  AcpiOsFree;
    ACPIGETCURRENTRESOURCES     AcpiGetCurrentResources;
    ACPIGETPOSSIBLERESOURCES    AcpiGetPossibleResources;
    ACPISETCURRENTRESOURCES     AcpiSetCurrentResources;
    ACPIENTERSLEEPSTATEPREP     AcpiEnterSleepStatePrep;
    ACPIENTERSLEEPSTATE         AcpiEnterSleepState;
    ACPIENTERSLEEPSTATES4BIOS   AcpiEnterSleepStateS4bios;
    ACPILEAVESLEEPSTATE         AcpiLeaveSleepState;
    ACPIUTSTRTOUL               AcpiUtStrtoul;
    SETTHROTTLINGORCSTATE       SetThrottlingOrCState;
    ACPIGETTIMERDURATION        AcpiGetTimerDuration;
    ACPIGETTIMER                AcpiGetTimer;
    ACPIWALKRESOURCES           AcpiWalkResources;
    ACPIUTMEMCPY                AcpiUtMemcpy;
    PSDAPPCOM                   PSD_APP_COM;
    FINDPCIDEVICE               FindPCIDevice;
} ACPIFUNCTION, *PACPIFUNCTION;

// Idle external hook
typedef UINT32 (*ACPIILEFUNCTION)(UINT32 State,volatile UINT32 LApic);
// SpeedStep/PowerNow! external hook
typedef ACPI_STATUS (*ACPIEXTHOOKPOWERSTATE)(UINT32 CPUnum,UINT32 State);
// Throttling/Q'n'Quit  external hook
typedef ACPI_STATUS (*ACPIEXTHOOKTHROTTLING)(UINT32 FlagGetSet,PTHROTTLING_C_STATE State);
// define for FlagGetSet
#define ACPI_EXTHOOK_SET_THROTTLING  0
#define ACPI_EXTHOOK_GET_THROTTLING  1

typedef struct _PsdSetup_
{
// This insert instead Rudi's dummy ULONG[6]
    ACPIILEFUNCTION         Idle;
    UINT32                  NewPSDStackSize; // stack for start ACPI, can be change /St:xxxxx
    UINT32                  CountCPU;        // Count init CPU
    P_F_2                   Router;
    UINT32                  UsableCPU;       // CPU in use
    UINT32                  IOAPICMask;      // IOAPICMask
//
    PACPIFUNCTION           Function;         // Pointer to ACPI function
// Bitfield options
    unsigned                SMP:1;            // 1 - /SMP is
    unsigned                PICOnly:1;        // 1 - /PIC is
    unsigned                ApicEnable:1;     // 1 - APICMODE
    unsigned                VirtualWired:1;   // 1 - Virtual Wired mode
    unsigned                InsidePSD:1;      // 0 - PSD call/ 1 - Device call
    unsigned                ACPIInit:1;       // 0 - not Init/ 1 - Init
    unsigned                IRQ0:1;           // For /TMR option
    unsigned                InstallOEM:1;     // 1 - need install OEMHLP$, 0 - no
    unsigned                ResetNeed:1;      // Use reset in PSD_RESET_MODE
    unsigned                ShutDownFlag:1;   // Flag, that shutdown is beginning, don't use helpers
    unsigned                NeedACPIOff:1;    // Flag, that need turn off ACPI before kernel's RealMode call
    unsigned                ThrtlOn:1;        // Throttling is set
    unsigned                CrazyDriver:1;    // For disable all high IRQ before going to advanced mode
    unsigned                InitComplete:1;   // Kernel call INIT_COMPLETE for all drivers
    unsigned                Sleep:1;          // Goto AcpiSleepState
    unsigned                EnableSMPExec:1;  // Allow exec via Idle/Busy at all CPU
    unsigned                SMPKernel:1;      // 0 - Uni, 1 - SMP kernels
    unsigned                SetAPIClink:1;    // Setting APIC IRQ link, 0 - at INIT_COMPLETE, 1 - in PCI scan
    unsigned                SetPCIasACPI:1;   // Setting  IRQ in PCI Config space from our PCI scan IRQ
//
    UINT32                  IntNum;           // Interrupt number for ACPI
    UINT32                  LeaveSleep;       //
    UINT32                  DummyVector;      //
    UINT32                  IRQMask;          // Mask for intterrupts controllers
    UINT32                  CountIRQLine;     // Line in IO APIC, < 45
    UINT32                  ThrtlValue;       // Current value for throttling
    UINT32                  PCIMask;          // PCI IRQ mask, filled by PCI scanner
    ACPIEXTHOOKPOWERSTATE   ExtPowerState;    // SpeedStep/PowerNow! external hook
    ACPIEXTHOOKTHROTTLING   ExtThrottling;    // Throttling/Q'n'Quit  external hook
    UINT32                  MaxCpu;           // /MCPU options
} PSDSETUP, *PPSDSETUP;


typedef struct _PSDDEBUG
{
    ULONG                   Flags;      // debugging level
    PUCHAR                  VideoAddr;  // address of video memory
    ULONG                   VideoPoint; // current write position
    PUCHAR                  MemAddr;    // address of memory buffer
    ULONG                   MemSize;    // size of memory buffer
    ULONG                   WritePos;   // current write position
    ULONG                   ReadPos;    // current read position
    ULONG                   MemMask;    // mask for size
} PSDDEBUG, *PPSDDEBUG;

typedef struct _AcpiExportedFunction_
{
    UINT32                  Size;
    ACPIFUNCTION            Table;
} ACPIEXPORTEDFUNCTION, *PACPIEXPORTEDFUNCTION;

ACPI_STATUS Do_PCT( ACPI_HANDLE ObjHandle );
ACPI_STATUS Do_CST( ACPI_HANDLE ObjHandle );
ACPI_STATUS Do_CPU( ACPI_HANDLE ObjHandle );
ACPI_STATUS Do_PDC( ACPI_HANDLE ObjHandle );
void  GoToSMPSleep(UINT32);
ACPI_STATUS GoToUniSleep(UINT32 SleepSt);
PACPIFUNCTION APIENTRY InitACPICall(char *Component);

#pragma pack()

#endif
