/* $Id: 1$ */
/** @oemhpci.h
  *
  *  OEMHLP interface for ACPI
  *
  * netlabs.org confidential
  *
  * Copyright (c) 2006 netlabs.org
  *
  * Author: Ruediger Ihle
  *
  * All Rights Reserved
  *
  */

#ifndef	__OEMHPCI_H
#define	__OEMHPCI_H


#pragma	pack(1)


// commands

#define	PCI_GetBiosInfo		0
#define	PCI_FindDevice		1
#define	PCI_FindClass		2
#define	PCI_ReadConfig		3
#define	PCI_WriteConfig		4


// parameter packets ....

typedef struct
{
  UCHAR		Command;		// PCI_GetBiosInfo
} PCI_GET_BIOSINFO_PARM;

typedef struct
{
  UCHAR		Command;		// PCI_FindDevice
  USHORT	DeviceID;
  USHORT	VendorID;
  UCHAR		Index;
} PCI_FIND_DEVICE_PARM;

typedef struct
{
  UCHAR		Command;		// PCI_FindClass
  ULONG		ClassCode;
  UCHAR 	Index;
} PCI_FIND_CLASS_PARM;

typedef struct
{
  UCHAR		Command;		// PCI_ReadConfig
  UCHAR		BusNumber;
  UCHAR		DevFunc;
  UCHAR		Register;
  UCHAR		Size;
} PCI_READ_CONFIG_PARM;

typedef struct
{
  UCHAR		Command;		// PCI_WriteConfig
  UCHAR		BusNumber;
  UCHAR		DevFunc;
  UCHAR		Register;
  UCHAR		Size;
  ULONG		Data;
} PCI_WRITE_CONFIG_PARM;

typedef union
{
  UCHAR			Command;
  PCI_GET_BIOSINFO_PARM	GetBiosInfo;
  PCI_FIND_DEVICE_PARM	FindDevice;
  PCI_FIND_CLASS_PARM	FindClass;
  PCI_READ_CONFIG_PARM	ReadConfig;
  PCI_WRITE_CONFIG_PARM	WriteConfig;
} PCI_PARM_PAKET;


// data packets ....

typedef struct				// PCI_GetBiosInfo
{
  UCHAR		ReturnCode;		// O.K. == 0
  UCHAR		HardwareMechanism;
  UCHAR		Version_Major;
  UCHAR		Version_Minor;
  UCHAR		MaxBus;
} PCI_BIOS_INFO;

typedef struct				// PCI_FindDevice, PCI_FindClass
{
  UCHAR		ReturnCode;		// O.K. == 0
  UCHAR		BusNumber;
  UCHAR		DevFunc;
} PCI_DEVICE_INFO;

typedef struct				// PCI_ReadConfig
{
  UCHAR		ReturnCode;		// O.K. == 0
  ULONG		Data;
} PCI_CONFIG_DATA;


typedef union
{
  UCHAR			ReturnCode;
  PCI_BIOS_INFO		GetBiosInfo;
  PCI_DEVICE_INFO	DeviceInfo;
  PCI_CONFIG_DATA	ConfigData;
} PCI_DATA_PAKET;


#pragma	pack()


//----------------------------------------------------------------------//
//	OEMHLP extension stuff						//
//----------------------------------------------------------------------//

// ASM functions
extern VOID APIENTRY	OemHlpXInitialize(VOID);
extern ULONG APIENTRY	OemHlpXInstallPDD(VOID);
extern ULONG APIENTRY	OemHlpXShareIRQ(ULONG ulFirstIRQ, ULONG ulLastIRQ);

extern ULONG APIENTRY	OemHlpXQueryAPI(ULONG ulEntryID);
extern VOID APIENTRY	OemHlpXIODelay(ULONG ulUnitsOf500ns);
extern ULONG APIENTRY	OemHlpXCallPDD(ULONG ulFunc, ...);
extern VOID APIENTRY	OemHlpXCtxHookStub(ULONG ulParam);

// Handler commands (special)
#define	PSDCMD_QUERYRESOURCES		0xff010000
#define	PSDCMD_PROCESSIRQ		0xff020000
#define	PSDCMD_BASEDEVINIT		0xff030000

// Handler commands (PDD strategy)
#define	PSDCMD_STRATEGY(func, sub)	((((func) & 0xff) << 16) | (sub & 0xff))

// Handler commands (PDD IOCTL)
#define	PSDCMD_OEMH_IOC(func)		(((func) & 0xff) | 0x00108000)
#define	PSDCMD_ACPI_IOC(func)		(((func) & 0xff) | 0x00108800)
#define	PSDCMD_TEST_IOC(func)		(((func) & 0xff) | 0x00109000)

// PSD interface
extern ULONG APIENTRY	OemHlpXHandler(ULONG ulFunc, PVOID pParam,
				       PVOID pData, ULONG ulLengthInfo);



//----------------------------------------------------------------------//
//	PCI config space access			  			//
//----------------------------------------------------------------------//

// make address description from components
#define	PCI_ADDR_DESCR(BusNo, DevFn, RegNo, Size)	\
	(((Size) << 24) | ((BusNo) << 16) | ((DevFn) << 8) | (RegNo))

// pointers to PCI access functions
extern BOOL (*OemHlpReadPCI)(ULONG ulAddrDesc, PVOID pvVal);
extern BOOL (*OemHlpWritePCI)(ULONG ulAddrDesc, PVOID pvVal);
extern ULONG OemHlpPCILastBus;

// replacement for PCI functions in original OEMHLP$
ULONG OemHlpCat80Func0b(PCI_PARM_PAKET *pParm, PCI_DATA_PAKET *pData);

// PS+++ Add DosTable struct for intSwitchStack
#pragma	pack(1)
//----------------------------------------------------------------------//
//	DosTable                                                         //
//----------------------------------------------------------------------//
typedef struct _DosTable_
{
    UCHAR            DosTableSize;         // size in dword
    ULONG            Body[1];              // Don't intrested now
} DOSTABLE, *PDOSTABLE;
//
typedef struct _DosTable2_
{
    UCHAR            DosTable2Size;        // size in dword
    ULONG            ErrMap24;
    ULONG            ErrMap24End;
    ULONG            Err_Table_24;
    ULONG            CDSAddr;
    ULONG            GDT_RDR1;
    ULONG            *InterruptLevel;
    ULONG            InDos;
    ULONG            RealModeStackSegment;
    ULONG            Seg3xBoxPTDA;
    ULONG            FlatCS;
    ULONG            FlatDS;
    ULONG            *TKSSBase;          // Variable containing stack base
    void             *intSwitchStack;    // private stack switch rtn
    UCHAR            *privateStack;      // interrupted linear stack
    ULONG            PhysDiskTablePtr;   // disk table
    ULONG            forceEMHandler;     // change NPX emul rtn 
    ULONG            ReserveVM;          // alloc virtual memory 
    ULONG            pgpPageDir;         // lin addr of dir table ;novell
} DOSTABLE2, *PDOSTABLE2;

/* --------------------------------------------------------------------------*/
typedef struct 
{
    UINT16           VendorID, DeviceID;                            // 00h
    UINT16           Command, Status;                               // 04h  
    UINT8            Revision;
    UINT8            ProgIF, SubClassCode, ClassCode;               // 08h
    UINT8            CacheLineSize, LatencyTimer, HeaderType, BIST; // 0Ch
    UINT32           BaseAddr[6];                                   // 10h
    UINT32           CIS;                                           // 28h
    UINT16           SubVendorID, SubDeviceID;                      // 2Ch
    UINT32           EPROMBase;                                     // 30h
    UINT8            CapPinter,reserved[3];                         // 34h
    UINT32           Reserved;                                      // 38h
    UINT8            IRQ, Int, MinGnt, MaxLat;                      // 3Ch
    UCHAR            x[0xC0];
} tPCIConfigArea, *pPCIConfigArea;

/* --------------------------------------------------------------------------*/

#define MAX_PCI_BUSES 256
#define MAX_PCI_DEVICES 32
#define MAX_PCI_FUNCTIONS 8

#define PCIREG_VENDOR_ID 0x00
#define PCIREG_DEVICE_ID 0x02
#define PCIREG_COMMAND 0x04
#define PCIREG_STATUS 0x06
#define PCIREG_REVISION 0x08
#define PCIREG_PROGIF 0x09
#define PCIREG_CLASS_CODE 0x0A
#define PCIREG_SUBCLASS 0x0A
#define PCIREG_CLASS 0x0B
#define PCIREG_CACHE_LINE 0x0C
#define PCIREG_LATENCY 0x0D
#define PCIREG_HEADER_TYPE 0x0E
#define PCIREG_PRIMARY_BUS 0x18
#define PCIREG_SECONDARY_BUS 0x19
#define PCIREG_SUBORDINATE_BUS 0x1A
#define PCIREG_CAP_LIST 0x34
#define PCIREG_CAP_LIST_CB 0x14
#define PCIREG_IRQ   0x3C
#define PCIREG_INT_PIN 0x3D

#define PCI_STATUS_CAP_LIST	0x10

#define PCI_CAP_ID_MSI  0x05	/* Message Signalled Interrupts */
#define PCI_CAP_ID_MSIX  0x11	/* MSI-X */
#define PCI_CAP_LIST_NEXT  1	/* Next capability in the list */
#define PCI_CAP_FLAGS   2	/* Capability defined flags (16 bits) */
#define PCI_MSI_FLAGS_ENABLE 0x01	/* MSI feature enabled */
#define PCI_MSIX_FLAGS_ENABLE 0x8000	/* MSI-X feature enabled */

#define PCI_HEADER_TYPE_NORMAL     0
#define PCI_HEADER_TYPE_BRIDGE     1
#define PCI_HEADER_TYPE_CARDBUS    2

#define PCI_CLASS_BRIDGE_HOST       0x0600
#define PCI_CLASS_BRIDGE_ISA        0x0601
#define PCI_CLASS_BRIDGE_PCI        0x0604
#define PCI_CLASS_BRIDGE_PCMCIA     0x0605
#define PCI_CLASS_BRIDGE_NUBUS      0x0606
#define PCI_CLASS_BRIDGE_CARDBUS    0x0607
#define PCI_CLASS_BRIDGE_RACEWAY    0x0608
#define PCI_CLASS_BRIDGE_SEMI       0x0609
#define PCI_CLASS_BRIDGE_FAKE       0x06FF

#define MAX_NUM_ADAPTERS 256
#define PCI_MAX_IRQS	    256

typedef struct _Adapter  tAdapter,  *pAdapter;
typedef struct _IRQLink  tIRQLink,  *pIRQLink;
typedef struct _IRQRoute tIRQRoute, *pIRQRoute;
typedef struct _Bridge   tBridge,   *pBridge;

typedef enum 
{ 
    PICmode = 0, 
    APICmode = 1 
}   tRouterModes;

typedef struct _Bitmap 
{
    UINT32 Bits [PCI_MAX_IRQS / 32];
}   tBitmap, *pBitmap;

struct _IRQLink 
{
    ACPI_HANDLE Handle;
    ACPI_NAME   Name;

    tBitmap     Possible[2];
    UINT8       Current[2];
    UINT8       SelectedAPIC;

    UINT8        evaluated;

    pAdapter    wired[2];
};

typedef enum _LinkType 
{ 
    GSI = 0, 
    LNK 
}   tLinkType;

struct _IRQRoute 
{
    UINT8     Dev;
    UINT8     Pin;
    tLinkType Type;
    union 
    {
         UINT8   IRQ;
         UINT8   Link;
    };
};

struct _Adapter 
{
    UINT8       Bus, Dev, Fnc;
    UINT8       Typ;
    UINT16      Vendor, Device;

    ACPI_HANDLE Handle;
    ACPI_NAME   Name;

    pIRQRoute   route[2];
    pAdapter    wired[2];

    pBridge     Bridge;

    UINT8       IntPin, PCIIRQ, PicIRQ, APicIRQ, MSI;
};

struct _Bridge 
{
    pAdapter    Adapter;

    USHORT      Type;
    SHORT       PriBus;
    UCHAR       SecBus, SubBus;

    unsigned    numRouterEntries[2];
    pIRQRoute   RouterTable[2];
};
/* --------------------------------------------------------------------------*/

// PCI BIOS 2.1 return codes
#define	BIOSRC_SUCCESSFUL		0
#define	BIOSRC_FUNC_NOT_SUPPORTED	0x81
#define	BIOSRC_BAD_VENDOR_ID		0x83
#define	BIOSRC_DEVICE_NOT_FOUND		0x86
#define	BIOSRC_BAD_REGISTER_NUMBER	0x87
#define	BIOSRC_SET_FAILED		0x88
#define	BIOSRC_BUFFER_TOO_SMALL		0x89

typedef struct _BridgeFound_
{
    ULONG             BBN;
    ULONG             Dev;
    ULONG             Fun;
} BridgeFound, *PBridgeFound;

struct 
{
    USHORT           Vendor;
    USHORT           Device;
} ChipID;

#define	PCIReadConfigB(b,d,f,r,p)		\
	OemHlpReadPCI(PCI_ADDR_DESCR(b,((d)<<3)|f,r,1), p)

#define	PCIReadConfigW(b,d,f,r,p)		\
	OemHlpReadPCI(PCI_ADDR_DESCR(b,((d)<<3)|f,r,2), p)

#define	PCIReadConfigL(b,d,f,r,p)		\
	OemHlpReadPCI(PCI_ADDR_DESCR(b,((d)<<3)|f,r,4), p)

/* --------------------------------------------------------------------------*/

#define PDDRET_USEORIGINAL 0x80000000

#endif


