/***************************************************************************
 *      Created 2005  eCo Software                                         *
 *                                                                         *
 *      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is        *
 *      sample code created by eCo Software. This sample code is not part  *
 *      of any standard or eCo Software product and is provided to you     *
 *      solely for the purpose of assisting you in the development of your *
 *      applications.  The code is provided "AS IS", without               *
 *      warranty of any kind. eCo Software shall not be liable for any     *
 *      damages arising out of your use of the sample code, even if they   *
 *      have been advised of the possibility of such damages.              *
 *-------------------------------------------------------------------------*/
#define INCL_DOSPROCESS
#include <os2.h>
#include "acpi.h"
#include "acpiapi.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "acpiIdlist.h"

char *AcpiStatusToStr(ACPI_STATUS);

typedef struct _GpeDevice_
{
    ACPI_HANDLE Device;
    UINT32      GpeNumber;
    UINT32      SxDState;
    char        PnPid[8];
    char        *PnPidInfo;
} GPEDEVICE, *PGPEDEVICE;

typedef struct _GPEEvent_
{
    UINT8       GpeUsed;
    UINT8       GpeTriggered;
    UINT8       WakeRintime;
    ACPI_HANDLE GpeNdl;
} GPEEVENT, *PGEPEEVENT;

GPEEVENT     Event[256];
GPEDEVICE    Device[256];
UINT32       DeviceCount = 0;;
ACPI_OBJECT  Object[20];
/**
* Callback for GPE scanning
* @ObjHandle    handle to GPE object
* @NestingLevel nesting level
* @Context      context
* @ReturnValue  return value
* @return       ACPI status
*/
ACPI_STATUS
GetGpeNotify(
    ACPI_HANDLE ObjHandle,
    UINT32 NestingLevel,
    void *Context,
    void **ReturnValue)
{
    ACPI_BUFFER Buffer;
    char        DevName[5];
    ULONG       GpeNumber, GpeTriggered, i, Flag;

    Buffer.Length  = 5;
    Buffer.Pointer = DevName;
    AcpiGetName(ObjHandle, ACPI_SINGLE_NAME, &Buffer);
    DevName[5]     = 0;

    if ((DevName[1] == 'L') || (DevName[1] == 'l'))
    {
         GpeTriggered = ACPI_GPE_LEVEL_TRIGGERED;
    }

    if ((DevName[1] == 'E') || (DevName[1] == 'e'))
    {
         GpeTriggered = ACPI_GPE_EDGE_TRIGGERED;
    }

    DevName[0] = '0';
    DevName[1] = 'X';
    GpeNumber  = strtoul(DevName,NULL,0);
    Event[GpeNumber].GpeNdl       = ObjHandle;
    Event[GpeNumber].GpeUsed      = 1;
    Event[GpeNumber].GpeTriggered = GpeTriggered;
    printf("GPE0x%2.2X triggered by %s \n",GpeNumber,
                                        Event[GpeNumber].GpeTriggered == ACPI_GPE_EDGE_TRIGGERED ? "Edge" : "Level"
          );

    Flag = 0;
    for (i = 0; i < DeviceCount; i++)
    {
         if (Device[i].GpeNumber != GpeNumber)
            continue;
         if (!Flag)
         {
             printf("        Using by device:\n\n");
             Flag++;
         }
         Buffer.Length  = 5;
         Buffer.Pointer = DevName;
         AcpiGetName(Device[i].Device, ACPI_SINGLE_NAME, &Buffer);
         DevName[5]     = 0;
         printf("        [%s] S%dD %s %s\n",DevName, Device[i].SxDState,Device[i].PnPid, Device[i].PnPidInfo == NULL? "" :Device[i].PnPidInfo);
    }
    if (Flag)
       printf("\n");

    return(AE_OK);
}
/*
 * CallBack function for _PSW
 */
ACPI_STATUS 
GetGpeDevice(
    ACPI_HANDLE ObjHandle,
    UINT32      NestingLevel,
    void        *Context,
    void        **ReturnValue)
{
    char        DevName[5];
    ACPI_BUFFER Results, Buffer;
    ACPI_OBJECT       *Obj;
    ACPI_STATUS     Status;
    ULONG           i,ndev;
    ACPI_DEVICE_INFO DevInfo;

    Results.Length  = sizeof(Object);
    Results.Pointer = Object;

    Status = AcpiEvaluateObject(ObjHandle,"_PRW",NULL,&Results);
    if (Status != AE_OK)
        return AE_OK;

    Buffer.Length  = 5;
    Buffer.Pointer = DevName;
    AcpiGetName(ObjHandle, ACPI_SINGLE_NAME, &Buffer);
    DevName[5]     = 0;

    Obj = (ACPI_OBJECT *)Object;
    if ((Obj->Type == ACPI_TYPE_PACKAGE) && (Obj->Package.Count == 2))
    {
        Device[DeviceCount].Device    = ObjHandle;
        Device[DeviceCount].GpeNumber = Obj->Package.Elements[0].Integer.Value;
        Device[DeviceCount].SxDState  = Obj->Package.Elements[1].Integer.Value;
        Device[DeviceCount].PnPidInfo = NULL;
        Device[DeviceCount].PnPid[0]  = '\0';

        Buffer.Length  = sizeof(DevInfo);
        Buffer.Pointer = &DevInfo;
        Status         = AcpiGetObjectInfo(ObjHandle, &Buffer);
        if (Status)
        {
            DeviceCount++;
            return AE_OK;
        }
        sprintf(Device[DeviceCount].PnPid,"%7.7s",&DevInfo.HardwareId);
        ndev = sizeof(HidCodeInfo)/sizeof(struct Hid_Code_Info);
        for (i = 0; i < ndev; i++)
        {   
             if (!stricmp((char *)&(DevInfo.HardwareId), HidCodeInfo[i].pHid) )
             {   
                 Device[DeviceCount].PnPidInfo = HidCodeInfo[i].Info; 
                 break;
             }
        }
        DeviceCount++;
    }

    return AE_OK;
}
int
main(
    void)
{
    ACPI_STATUS     Status;
    ACPI_HANDLE     GpeHdl;


 
    Status = AcpiWalkNamespace(ACPI_TYPE_DEVICE, // All events
                               ACPI_ROOT_OBJECT, // Start object
                               ACPI_UINT32_MAX,  // Max depth
                               GetGpeDevice,     // Callback
                               NULL, NULL);      // Callback param and poitner to return value

    memset (Event ,0 ,sizeof (GPEEVENT) * 256);
    Status = AcpiGetHandle (ACPI_ROOT_OBJECT,"_GPE",&GpeHdl);
    if (Status == AE_OK)
    {
        Status = AcpiWalkNamespace(ACPI_TYPE_ANY,    // All events
                                   GpeHdl,           // Start object
                                   ACPI_UINT32_MAX,  // Max depth
                                   GetGpeNotify,     // Callback
                                   &GpeHdl, NULL);   // Callback param and poitner to return value
        printf("EndScan GPE Status %x\n",Status);
    }
    else
    {
        printf("Get GpeHdl Status = 0x%x\n",Status);
        return 1;
    }

    return 0;
}
